==> emelfm2-0.1.2.tar.gz: emelfm2-0.1.2/plugins/e2p_glob.c <==
/*
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_plugins.h"
#include "e2_dialog.h"
#include "e2_filelist.h"

static GtkWidget *pattern_entry;
static GtkWidget *invert_check;
static GtkWidget *case_sensitive_check;

static void _e2p_glob_no_cb (GtkWidget *widget, GtkWidget *dialog)
{
  gtk_widget_set_sensitive (app.main_window, TRUE);
  gtk_widget_grab_focus (curr_view->treeview);
  gtk_widget_destroy (dialog);
  gtk_main_quit();
}

static gboolean _e2p_glob_key_press_cb (GtkWidget *widget,
	GdkEventKey *event, GtkWidget *dialog)
{
  if (event->keyval == GDK_Escape)
  {
	_e2p_glob_no_cb (NULL, dialog);
	return TRUE;
  }
  return FALSE;
}

static void _e2p_glob_yes_cb (GtkWidget *widget, GtkWidget *dialog)
{
  gchar *pattern, *s, *free_this, *utf;
  GList *patterns = NULL;
  gboolean invert, case_sensitive;

  if ((s = (gchar *) gtk_entry_get_text(GTK_ENTRY(pattern_entry))) == NULL)
    return;

  invert = GTK_TOGGLE_BUTTON(invert_check)->active;
  case_sensitive = GTK_TOGGLE_BUTTON(case_sensitive_check)->active;

  free_this = pattern = g_strdup (s);
  if (!case_sensitive)
    pattern = g_utf8_strdown (pattern, strlen (pattern));
  while ((s = strchr (pattern, ',')) != NULL)	//if always ascii ',', don't need g_utf8_strchr()
  {
    *s = '\0';
    patterns = g_list_append(patterns, pattern);
    pattern = s+1;
  }
  patterns = g_list_append(patterns, pattern);
  GtkTreeSelection *selection = gtk_tree_view_get_selection
    (GTK_TREE_VIEW (curr_view->treeview));
  gtk_tree_selection_unselect_all (selection);
  //iterate over all iters in current view
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (curr_view->treeview));
  GtkTreeIter iter;
  gboolean isrow = gtk_tree_model_get_iter_first (model, &iter);
  FileInfo *info;
  gchar *thisname;
  F_DECLARE;

  while (isrow)
  {
  	gtk_tree_model_get (model, &iter, FINFO, &info, -1);
  	if (case_sensitive)
		thisname = D_FILENAME_FROM_LOCALE (info->filename);
    else
	{
		utf = F_FILENAME_FROM_LOCALE (info->filename);
		thisname = g_utf8_strdown (utf, -1);
		F_FREE (utf);
	}
  	GList *tmp;
  	for (tmp = patterns; tmp != NULL; tmp = tmp->next)
  	{
		pattern = tmp->data;
		if (g_pattern_match_simple (pattern, thisname))
		{
			if (!invert)
			{
				gtk_tree_selection_select_iter (selection, &iter);
				break;
			}
		}
		else if (invert)
		{
			gtk_tree_selection_select_iter (selection, &iter);
			break;
		}
	}
	g_free (thisname);
	isrow = gtk_tree_model_iter_next (model, &iter);
  }

  g_list_free (patterns);
  g_free (free_this);
  
  return _e2p_glob_no_cb (widget, dialog);
}

static void _e2p_glob (void)
{
  GtkWidget *dialog;
  GtkWidget *dialog_vbox;
  GtkWidget *table;
  GtkTooltips *tooltips;

  dialog = gtk_dialog_new();
  E2_DIALOG_TITLE (GTK_WINDOW (dialog), _("select matching files"));
  dialog_vbox = GTK_DIALOG (dialog)->vbox;
  gtk_container_set_border_width (GTK_CONTAINER(dialog_vbox), E2_PADDING);
  gtk_box_set_spacing (GTK_BOX (dialog_vbox), E2_PADDING);
  g_signal_connect (G_OBJECT (dialog), "key_press_event",
	G_CALLBACK (_e2p_glob_key_press_cb), dialog);
  tooltips = gtk_tooltips_new();

  e2_widget_add_label (dialog_vbox, _("Glob Select: "), 0.0, FALSE, E2_PADDING);

  pattern_entry = add_entry(dialog_vbox, "*", FALSE, 0);
  g_signal_connect (G_OBJECT (pattern_entry), "activate",
	G_CALLBACK (_e2p_glob_yes_cb), dialog);
  gtk_widget_grab_focus (pattern_entry);

  GtkTreeSelection *selection = gtk_tree_view_get_selection
    (GTK_TREE_VIEW (curr_view->treeview));
  if (gtk_tree_selection_count_selected_rows (selection) > 0)
  {
  	GString *text;
  	FileInfo *info = e2_fileview_get_selected_first_local (curr_view);
	gchar *s;
    F_DECLARE;
  	gchar *utf = F_FILENAME_FROM_LOCALE (info->filename);
    if ((s = strrchr (utf, '.')) != NULL)
    {
      text = g_string_new (s);
      g_string_prepend_c (text, '*');
      gtk_entry_set_text (GTK_ENTRY(pattern_entry), text->str);
      gtk_editable_select_region (GTK_EDITABLE(pattern_entry), 0, -1);
      g_string_free (text, TRUE);
    }
	F_FREE (utf);
  }

  e2_widget_add_label (dialog_vbox, _("Example: *.c,*.h"), 0.0, FALSE, 0);

  table = add_table(dialog_vbox, 1, 2, FALSE, FALSE, 0);
  invert_check = e2_button_add_toggle_to_table(table, _("Invert"),
		FALSE, NULL, NULL, 0, 1, 0, 1);
  case_sensitive_check = e2_button_add_toggle_to_table(table, _("Case Sensitive"),
		TRUE, NULL, NULL, 1, 2, 0, 1);
  gtk_tooltips_set_tip( GTK_TOOLTIPS (tooltips), invert_check,
		_("Select files that DO NOT match the given mask"), NULL);

  e2_dialog_add_button_custom (dialog, FALSE, &E2_BUTTON_CANCEL,
	  NULL, _e2p_glob_no_cb, dialog);
  e2_dialog_add_button_custom (dialog, TRUE, &E2_BUTTON_OK,
	  NULL, _e2p_glob_yes_cb, dialog);
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
  gtk_widget_show (dialog);
  gtk_widget_set_sensitive (app.main_window, FALSE);
  gtk_main ();
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
#define ANAME "glob"
  aname = _("glob");

  p->signature = ANAME VERSION;
  p->menu_name = _("_Glob..");
  p->description = _("Select items matching a specified pattern");
  p->icon = ICON_DIR"plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate

  if (p->plugin_cb == NULL)
  {
	  p->plugin_cb = _e2p_glob;  //set plugins menu item function
//	  E2_Action* (*action_register) () = (*p->api_lookup) (E2API_ACTION_REGISTER);
	  //no need to free this
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
	  //NULL = no data, FALSE = no argument for the function
//	  (*action_register)
	  e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2p_glob, NULL, FALSE);
	  return TRUE;
  }
  return FALSE;
}
/**
 * @brief unload this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return
 **/
void unload_plugin (Plugin *p)
{
//  if (p->api_lookup != NULL)
//  {
//	  gboolean (*action_unregister) () = (*p->api_lookup) (E2API_ACTION_UNREGISTER);
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
//	  (*action_unregister)
	  e2_action_unregister (action_name);
	  g_free (action_name);
//  }
}

==> emelfm2-0.1.4.tar.gz: emelfm2-0.1.4/plugins/e2p_glob.c <==
/* $Id: e2p_glob.c 1346 2005-10-26 23:26:58Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_plugins.h"
#include "e2_dialog.h"
#include "e2_filelist.h"

static GtkWidget *pattern_entry;
static GtkWidget *invert_check;
static GtkWidget *case_sensitive_check;

static void _e2p_glob_no_cb (GtkWidget *widget, GtkWidget *dialog)
{
  gtk_widget_set_sensitive (app.main_window, TRUE);
  gtk_widget_grab_focus (curr_view->treeview);
  gtk_widget_destroy (dialog);
  gtk_main_quit();
}

static gboolean _e2p_glob_key_press_cb (GtkWidget *widget,
	GdkEventKey *event, GtkWidget *dialog)
{
  if (event->keyval == GDK_Escape)
  {
	_e2p_glob_no_cb (NULL, dialog);
	return TRUE;
  }
  return FALSE;
}

static void _e2p_glob_yes_cb (GtkWidget *widget, GtkWidget *dialog)
{
  gchar *pattern, *s, *free_this, *utf;
  GList *patterns = NULL;
  gboolean invert, case_sensitive;

  if ((s = (gchar *) gtk_entry_get_text(GTK_ENTRY(pattern_entry))) == NULL)
    return;

  invert = GTK_TOGGLE_BUTTON(invert_check)->active;
  case_sensitive = GTK_TOGGLE_BUTTON(case_sensitive_check)->active;

  free_this = pattern = g_strdup (s);
  if (!case_sensitive)
    pattern = g_utf8_strdown (pattern, strlen (pattern));
  while ((s = strchr (pattern, ',')) != NULL)	//if always ascii ',', don't need g_utf8_strchr()
  {
    *s = '\0';
    patterns = g_list_append(patterns, pattern);
    pattern = s+1;
  }
  patterns = g_list_append(patterns, pattern);
  GtkTreeSelection *sel = curr_view->selection;
  gtk_tree_selection_unselect_all (sel);
  //iterate over all iters in current view
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (curr_view->treeview));
  GtkTreeIter iter;
  gboolean isrow = gtk_tree_model_get_iter_first (model, &iter);
  FileInfo *info;
  gchar *thisname;

  while (isrow)
  {
  	gtk_tree_model_get (model, &iter, FINFO, &info, -1);
  	if (case_sensitive)
		thisname = D_FILENAME_FROM_LOCALE (info->filename);
    else
	{
		utf = F_FILENAME_FROM_LOCALE (info->filename);
		thisname = g_utf8_strdown (utf, -1);
		F_FREE (utf);
	}
  	GList *tmp;
  	for (tmp = patterns; tmp != NULL; tmp = tmp->next)
  	{
		pattern = tmp->data;
		if (g_pattern_match_simple (pattern, thisname))
		{
			if (!invert)
			{
				gtk_tree_selection_select_iter (sel, &iter);
				break;
			}
		}
		else if (invert)
		{
			gtk_tree_selection_select_iter (sel, &iter);
			break;
		}
	}
	g_free (thisname);
	isrow = gtk_tree_model_iter_next (model, &iter);
  }

  g_list_free (patterns);
  g_free (free_this);
  
  return _e2p_glob_no_cb (widget, dialog);
}

static void _e2p_glob (void)
{
  GtkWidget *dialog;
  GtkWidget *dialog_vbox;
  GtkWidget *table;
  GtkTooltips *tooltips;

  dialog = gtk_dialog_new();
  E2_DIALOG_TITLE (GTK_WINDOW (dialog), _("select matching files"));
  dialog_vbox = GTK_DIALOG (dialog)->vbox;
  gtk_container_set_border_width (GTK_CONTAINER(dialog_vbox), E2_PADDING);
  gtk_box_set_spacing (GTK_BOX (dialog_vbox), E2_PADDING);
  g_signal_connect (G_OBJECT (dialog), "key_press_event",
	G_CALLBACK (_e2p_glob_key_press_cb), dialog);
  tooltips = gtk_tooltips_new();

  e2_widget_add_label (dialog_vbox, _("Glob Select: "), 0.0, FALSE, E2_PADDING);

  pattern_entry = add_entry(dialog_vbox, "*", FALSE, 0);
  g_signal_connect (G_OBJECT (pattern_entry), "activate",
	G_CALLBACK (_e2p_glob_yes_cb), dialog);
  gtk_widget_grab_focus (pattern_entry);

  GtkTreeSelection *sel = curr_view->selection;
  if (gtk_tree_selection_count_selected_rows (sel) > 0)
  {
  	GString *text;
  	FileInfo *info = e2_fileview_get_selected_first_local (curr_view);
	gchar *s;
  	gchar *utf = F_FILENAME_FROM_LOCALE (info->filename);
    if ((s = strrchr (utf, '.')) != NULL)
    {
      text = g_string_new (s);
      g_string_prepend_c (text, '*');
      gtk_entry_set_text (GTK_ENTRY(pattern_entry), text->str);
      gtk_editable_select_region (GTK_EDITABLE(pattern_entry), 0, -1);
      g_string_free (text, TRUE);
    }
	F_FREE (utf);
  }

  e2_widget_add_label (dialog_vbox, _("Example: *.c,*.h"), 0.0, FALSE, 0);

  table = add_table(dialog_vbox, 1, 2, FALSE, FALSE, 0);
  invert_check = e2_button_add_toggle_to_table(table, _("Invert"),
		FALSE, NULL, NULL, 0, 1, 0, 1);
  case_sensitive_check = e2_button_add_toggle_to_table(table, _("Case Sensitive"),
		TRUE, NULL, NULL, 1, 2, 0, 1);
  gtk_tooltips_set_tip( GTK_TOOLTIPS (tooltips), invert_check,
		_("Select files that DO NOT match the given mask"), NULL);

  e2_dialog_add_button_custom (dialog, FALSE, &E2_BUTTON_CANCEL,
	  NULL, _e2p_glob_no_cb, dialog);
  e2_dialog_add_button_custom (dialog, TRUE, &E2_BUTTON_OK,
	  NULL, _e2p_glob_yes_cb, dialog);
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
  gtk_widget_show (dialog);
  gtk_widget_set_sensitive (app.main_window, FALSE);
  gtk_main ();
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
#define ANAME "glob"
  aname = _("glob");

  p->signature = ANAME VERSION;
  p->menu_name = _("_Glob..");
  p->description = _("Select items matching a specified pattern");
  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate

  if (p->plugin_cb == NULL)
  {
	  p->plugin_cb = _e2p_glob;  //set plugins menu item function
//	  E2_Action* (*action_register) () = (*p->api_lookup) (E2API_ACTION_REGISTER);
	  //no need to free this
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
	  //NULL = no data, FALSE = no argument for the function
//	  (*action_register)
	  e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2p_glob, NULL, FALSE);
	  return TRUE;
  }
  return FALSE;
}
/**
 * @brief unload this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return
 **/
void unload_plugin (Plugin *p)
{
//  if (p->api_lookup != NULL)
//  {
//	  gboolean (*action_unregister) () = (*p->api_lookup) (E2API_ACTION_UNREGISTER);
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
//	  (*action_unregister)
	  e2_action_unregister (action_name);
	  g_free (action_name);
//  }
}
